@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -47,6 +47,7 @@ mxTryHard <- function(
 	else if( !(0 %in% OKstatuscodes) ){OKstatuscodes <- c(OKstatuscodes,0)}
 	#if( !("MxModel" %in% class(model)) ){stop("argument 'model' must be an object of class 'MxModel'")}
 	if(initialTolerance<0){stop("value for argument 'initialTolerance' cannot be negative")}
+  warnModelCreatedByOldVersion(model)
 	if (omxHasDefaultComputePlan(model)) {
 		model@compute <- NULL
 	}
@@ -107,9 +108,12 @@ mxTryHard <- function(
 		RE=mxComputeReportExpectation()))
 	model@compute@.persist <- TRUE
 	modelAtStartValues <- suppressWarnings(try(runWithCounter(model, 0, silent, F)))
-	if(class(modelAtStartValues) != "try-error"){ 
+	if (!inherits(modelAtStartValues, "try-error")) {
 		fitvalAtStarts <- modelAtStartValues@fitfunction@result[1]
-		if(is.finite(fitvalAtStarts)){lowestminsofar <- fitvalAtStarts}
+		#If there are MxConstraints, we don't know if they're satisfied at the start values,
+		#so we don't want to treat the fit at the start values as the lowest so far,
+		#since an uphill step may be necessary to get to feasibility:
+		if(is.finite(fitvalAtStarts) && lackOfConstraints){lowestminsofar <- fitvalAtStarts}
 	}
 	model@compute <- inputCompute
 	rm(modelAtStartValues, inputCompute)
@@ -203,7 +207,7 @@ mxTryHard <- function(
 		
 		
 		#If fit resulted in error:
-		if( class(fit) == "try-error" || !is.finite(fit@fitfunction@result[1]) || fit$output$status$status== -1){
+		if (inherits(fit, "try-error") || !is.finite(fit@fitfunction@result[1]) || fit$output$status$status== -1) {
 			#^^^is.finite() returns FALSE for Inf, -Inf, NA, and NaN
 			lastBestFitCount <- 0
 			lastNoError<-FALSE
@@ -213,7 +217,7 @@ mxTryHard <- function(
 		
 		
 		#If fit did NOT result in error:
-		if(class(fit) != "try-error" && is.finite(fit@fitfunction@result[1]) && fit$output$status$status != -1){
+		if (!inherits(fit, "try-error") && is.finite(fit@fitfunction@result[1]) && fit$output$status$status != -1) {
 			lastNoError <- TRUE
 			validcount <- validcount + 1
 			if(fit@fitfunction@result[1] >= lowestminsofar){
@@ -258,12 +262,12 @@ mxTryHard <- function(
 				if(checkHess==TRUE) {
 					fit@output["infoDefinite"] <- TRUE
 					hessEigenval <- try(eigen(fit$output$calculatedHessian, symmetric = T, only.values = T)$values,silent=T)
-					if(class(hessEigenval)=='try-error') {
+					if (inherits(hessEigenval, 'try-error')) {
 						if(!silent){message(paste0(' Eigenvalues of Hessian could not be calculated'))}
 						goodflag <- FALSE
 						fit@output["infoDefinite"] <- FALSE
 					}
-					if(class(hessEigenval)!='try-error' && any(hessEigenval < 0)) {
+					if (!inherits(hessEigenval, 'try-error') && any(hessEigenval < 0)) {
 						if(!silent){message(paste0(' Not all eigenvalues of the Hessian are positive: ', paste(hessEigenval,collapse=', ')))}
 						goodflag <- FALSE
 						fit@output["infoDefinite"] <- FALSE
@@ -323,7 +327,7 @@ mxTryHard <- function(
 				finalfit <- OpenMx::mxModel(finalfit,mxComputeSequence(steps=steps))
 			}
 			finalfit <- suppressWarnings(try(runWithCounter(finalfit, numdone, silent, intervals=doIntervals)))
-			if(class(finalfit) == "try-error" || finalfit$output$status$status== -1) {
+			if (inherits(finalfit, "try-error") || finalfit$output$status$status== -1) {
 				if(!silent){message(' Errors during final fit for Hessian/SEs/CIs\n')}
 			} else {
 				if (length(summary(finalfit)$npsolMessage) > 0){
@@ -376,7 +380,7 @@ mxTryHard <- function(
 					finalfit <- OpenMx::mxModel(bestfit,mxComputeSequence(steps=steps))
 				}
 				finalfit <- suppressWarnings(try(runWithCounter(finalfit, numdone, silent, intervals=doIntervals)))
-				if(class(finalfit) == "try-error" || finalfit$output$status$status== -1) {
+				if (inherits(finalfit, "try-error") || finalfit$output$status$status== -1) {
 					if(!silent){message('Errors occurred during final run for Hessian/SEs/CIs; returning best fit as-is\n')}
 				}
 			}
@@ -407,7 +411,7 @@ mxTryHard <- function(
 	}
 	
 	if (!exists("bestfit")) {
-		if(class(fit) == 'try-error') warning(fit[[length(fit)]])
+		if (inherits(fit, 'try-error')) warning(fit[[length(fit)]])
 		imxReportProgress("", previousLen)
 		message('\n All fit attempts resulted in errors - check starting values or model specification\n')
 		bestfit<-fit
@@ -444,6 +448,7 @@ imxJiggle <- function(params, lbounds, ubounds, dsn, loc, scale){
 
 
 mxJiggle <- function(model, classic=FALSE, dsn=c("runif","rnorm","rcauchy"), loc=1, scale=0.25){
+  warnModelCreatedByOldVersion(model)
 	dsn <- match.arg(dsn, c("runif","rnorm","rcauchy"))
 	loc <- as.numeric(loc[1])
 	scale <- as.numeric(scale[1])
@@ -522,6 +527,14 @@ THFrankenmodel <- function(finalfit,bestfit,defaultComputePlan,Hesslater,SElater
 	bestfit@output$independentTime <- bestfit@output$independentTime + finalfit@output$independentTime
 	bestfit@output$wallTime <- bestfit@output$wallTime + finalfit@output$wallTime
 	bestfit@output$cpuTime <- bestfit@output$cpuTime + finalfit@output$cpuTime
+	needednames <- names(finalfit@output)[which(!(names(finalfit@output) %in% names(bestfit@output)))]
+	#Whether or not output elements having to do with CIs, SEs, or Hessians should be returned is governed
+	#by user-provided arguments to mxTryHard(); those elements are handled by code above:
+	needednames <- needednames[
+		!(needednames %in% c(
+			"confidenceIntervals","confidenceIntervalCodes","calculatedHessian","hessian","standardErrors","infoDefinite","conditionNumber","vcov"))
+	]
+	bestfit@output[needednames] <- finalfit@output[needednames]
 	bestfit@.modifiedSinceRun <- FALSE
 	return(bestfit)
 }
