@@ -3,6 +3,10 @@
 set.seed(1)
 
 library(OpenMx)
+library(testthat)
+
+if (!imxHasOpenMP()) stop("SKIP")
+
 N <- 2000
 u <- rbinom(N,1,.5)
 x <- .5*u+rnorm(N)
@@ -15,12 +19,16 @@ model <- mxModel( 'BinCont',
   mxAlgebra( Means + Betas%x%U , name='eMean'),
   mxMatrix('Full',nrow=1,ncol=1,free=T,name='Thresh'),
   mxMatrix('Symm',nrow=2,ncol=2,free=c(T,T,F),values=c(1,0,1),name='Cov'),
-  mxData( data.frame(x,y,u) , type='raw'),
   mxExpectationNormal( means='eMean', covariance='Cov',thresholds='Thresh',threshnames='y',dimnames=c('x','y') ),
   mxFitFunctionML(vector=TRUE)
 )                                        
 
-maxThreads <- 6
+expect_error(mxGenerateData(model, 100),
+             "no data is available")
+
+model <- mxModel(model, mxData( data.frame(x,y,u) , type='raw'))
+
+maxThreads <- imxGetNumThreads()
 rowLik <- matrix(NA, N, maxThreads)
 for (rep in 1:maxThreads) {
   model <- mxOption(model, "Number of Threads", rep)  # failed with 3 or more!
