@@ -199,28 +199,39 @@ bool omxMatrix::canDiscard()
 	return true;
 }
 
+void omxMatrix::disconnect()
+{
+	if(algebra) {
+		omxFreeAlgebraArgs(algebra);
+		algebra = NULL;
+	}
+
+	if(fitFunction) {
+		delete fitFunction;
+		fitFunction = NULL;
+	}
+}
+
 void omxFreeMatrix(omxMatrix *om) {
 
     if(om == NULL) return;
 
 	omxFreeInternalMatrixData(om);
 
-	if(om->algebra != NULL) {
-		omxFreeAlgebraArgs(om->algebra);
-		om->algebra = NULL;
-	}
-
-	if(om->fitFunction != NULL) {
-		delete om->fitFunction;
-		om->fitFunction = NULL;
-	}
-
-	if (om->freeColnames) for (auto cn : om->colnames) free((void*)cn);
-	if (om->freeRownames) for (auto rn : om->rownames) free((void*)rn);
+  om->disconnect();
+  om->clearDimnames();
 
 	if (!om->hasMatrixNumber) delete om;
 }
 
+void omxMatrix::clearDimnames()
+{
+	if (freeColnames) { for (auto cn : colnames) free((void*)cn); freeColnames=false; }
+	if (freeRownames) { for (auto rn : rownames) free((void*)rn); freeRownames=false; }
+  colnames.clear();
+  rownames.clear();
+}
+
 /**
  * Copies an omxMatrix to a new R matrix object
  *
